@@ -15,12 +15,12 @@ Usage:
 import os
 import shlex
 from .ssh_utils.utils import run_cmd,get_print_sudo_cmd,run_local_cmd,run_remote_cmd
-from .file_utils.file_utils.type_checks import is_file,is_dir
+from .file_utils.file_utils.type_checks import is_file,is_dir,get_user_pass_host_key,is_exists
 from .abstract_classes import run_pruned_func
 _FILE_PATH_KEYS = ['file', 'filepath', 'file_path', 'path', 'directory', 'f', 'dst', 'dest']
 _CONTENTS_KEYS = ['cont', 'content', 'contents', 'data', 'datas', 'dat', 'src', 'source']
 
-
+            
 # --- Helper utilities --------------------------------------------------------
 def string_in_keys(strings, kwargs):
     """Find a matching keyword in kwargs that contains any of the given substrings."""
@@ -30,25 +30,60 @@ def string_in_keys(strings, kwargs):
                 return key
     return None
 
-
-def get_path(paths):
+def make_dirs(path,exist_ok=True,**kwargs):
+    if exist_ok or (not exist_ok and not is_dir(path,**kwargs)):
+        if get_user_pass_host_key(**kwargs):
+            kwargs['cmd']=f"mkdir {path}"
+            run_cmd(**kwargs)
+        else:
+            os.makedirs(path,exist_ok=exist_ok)
+    return path
+def path_join(*args):
+    path = None
+    for i,arg in enumerate(args):
+        if arg:
+            if i == 0:
+                path = arg
+            else:
+                path = os.path.join(path,arg)
+    return path
+def make_path(path,home_dir=None,file=None,**kwargs):
+    if path:
+        basename = os.path.basename(path)
+        parts = path.split('/')
+        parts = [part for part in parts if part]
+        
+        full_dir = home_dir or ''
+        if file == True or (file == None and ('.' in basename)):
+            pieces = parts[:-1] if len(parts) > 1 else []
+        else:
+            pieces=parts
+            basename=None
+        for piece in pieces:
+            full_dir = os.path.join(full_dir,piece)
+            make_dirs(full_dir,exist_ok=True,**kwargs)
+        if basename:
+            full_dir=path_join(full_dir,basename)
+        print(f"full_dir == {full_dir}")
+        return full_dir
+def get_path(paths,**kwargs):
     """Return the first valid path among given paths."""
     for path in paths:
         if isinstance(path, str):
-            if os.path.isfile(path):
+            if is_file(path,**kwargs):
                 return path
             dirname = os.path.dirname(path)
-            if os.path.exists(dirname):
+            if is_exists(dirname):
                 return path
     return None
 
 
-def break_down_find_existing(path):
+def break_down_find_existing(path,**kwargs):
     """Return the first non-existent subpath within a path chain."""
     test_path = ''
     for part in path.split(os.sep):
         test_path = os.path.join(test_path, part)
-        if not os.path.exists(test_path):
+        if not is_exists(test_path):
             return test_path if test_path else None
     return test_path
 
@@ -97,8 +132,8 @@ def write_to_path(
 
     # shell command that fully overwrites
     # (no append, replaces contents entirely)
-    base_cmd = f"echo {quoted_data} > {quoted_path}"
-
+    base_cmd = f'sudo sh -c "echo {quoted_data} > {quoted_path}"'
+    input(base_cmd)
     # optional sudo password injection
     full_cmd = get_print_sudo_cmd(
         cmd=base_cmd,
@@ -150,29 +185,36 @@ def write_to_file(*args, **kwargs):
     Returns the file_path written.
     """
     file_path, contents = check_read_write_params(*args, **kwargs)
+    dirname = os.path.dirname(file_path)
+    
     if contents is None:
         raise ValueError("Missing contents to write.")
     user_at_host = kwargs.get("user_at_host")
-    if user_at_host:
+    if get_user_pass_host_key(**kwargs):
+        make_dirs(dirname, exist_ok=True,**kwargs)
         kwargs["cwd"] = kwargs.get('cwd') or os.path.dirname(file_path)
         # sanitize for shell safety
         quoted_path = shlex.quote(file_path)
         quoted_data = shlex.quote(str(contents))
         # shell command that fully overwrites
         # (no append, replaces contents entirely)
-        kwargs["cmd"] = f"echo {quoted_data} > {quoted_path}"
-        return run_pruned_func(run_cmd,**kwargs)
+        kwargs["cmd"] = f'sh -c "echo {quoted_data} > {quoted_path}"'
+        if not kwargs.get('password') and not kwargs.get('key'):
+            kwargs["cmd"]=f'sudo {kwargs["cmd"]}'
+        result = run_pruned_func(run_cmd,**kwargs)
+        if not is_file(file_path,**kwargs) or str(contents) != read_from_file(file_path,**kwargs):
+            kwargs["cmd"]=f'sudo {kwargs["cmd"]}'
+            result = run_pruned_func(run_cmd,**kwargs)
+        return result
 
-
-    os.makedirs(os.path.dirname(file_path) or ".", exist_ok=True)
+    make_dirs(dirname or ".", exist_ok=True)
     with open(file_path, "w", encoding="utf-8") as f:
         f.write(str(contents))
     return file_path
 
 
 def read_from_file(file_path,**kwargs):
-    user_at_host = kwargs.get("user_at_host")
-    if user_at_host:
+    if get_user_pass_host_key(**kwargs):
         kwargs["cwd"] = kwargs.get('cwd') or os.path.dirname(file_path)
         basename = os.path.basename(file_path)
         kwargs["cmd"] = f'cat {basename}'
@@ -180,7 +222,36 @@ def read_from_file(file_path,**kwargs):
     """Read text content from a file."""
     with open(file_path, "r", encoding="utf-8") as f:
         return f.read()
-
+def get_rel_path(src,src_rel,directory):
+    if src.startswith(src_rel):
+        src = src[len(src_rel):]
+        rel_path = os.path.join(directory,src)
+        return rel_path
+def make_relative_path(src,src_rel,dst,**kwargs):
+    print(f"src == {src}\nsrc_rel == {src_rel}\dst == {dst}")
+    if src.startswith(src_rel):
+        rel_path = get_rel_path(src,src_rel,directory)
+        path = make_path(src,home_dir=rel_path,**kwargs)
+        print(f"path == {path}")
+        return path
+def copy_dirs(dirs,dst,src_rel=None,**kwargs):
+    for src in dirs:
+        if rel_path:
+            dst = make_relative_path(src,src_rel,dst,**kwargs)
+        make_path(dst,**kwargs)
+        
+def copy_file(src,dst,rel_path=None,**kwargs):
+    if rel_path:
+        dst = make_relative_path(src,rel_path,dst,**kwargs)
+    if get_user_pass_host_key(**kwargs):
+        contents=read_from_file(src,**kwargs)
+        write_to_file(contents=contents,file_path=dst,**kwargs)
+    else:
+        shutil.copy(src,dst)
+    return dst
+def copy_files(files,dst,rel_path=None,**kwargs):
+    for file in files:
+        copy_file(src=file,dst=dst,rel_path=rel_path,**kwargs)
 
 def create_and_read_file(*args, **kwargs):
     """
