@@ -143,6 +143,29 @@ def clean_all_imports(text=None,file_path=None,import_pkg_js=None):
     import_pkg_js["nulines"]=nu_lines
     return import_pkg_js
 
+def get_all_real_imps(file):
+    contents = read_from_file(file)
+    lines = contents.split('\n')
+    for line in lines:
+        if line.startswith('from '):
+            from_line = line.split('from ')[-1]
+            dot_fro = ""
+            dirname = file
+            for char in from_line:
+                if  char != '.':
+                    line = f"from {dot_fro}{eatAll(from_line,'.')}"
+                    if line in all_imps:
+                        line = ""
+                    break
+                if dot_fro == "":
+                    dot_fro = ""
+                dirname = os.path.dirname(dirname)
+                dirbase = os.path.basename(dirname)
+                dot_fro = f"{dirbase}.{dot_fro}"
+        if line:
+            all_imps.append(line)
+
+    return '\n'.join(all_imps)
 def save_cleaned_imports(text=None,file_path=None,write=False,import_pkg_js=None):
     import_pkg_js=get_all_imports(text=text,file_path=file_path,import_pkg_js=import_pkg_js)
     import_pkg_js = clean_all_imports(text=text,file_path=file_path,import_pkg_js=import_pkg_js)
